@@ -35,22 +35,22 @@ typedef enum
 	CLASS_SSEUP,
 } X64Class;
 
-static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs, Registers *needed_registers, NamedArgument is_named);
+static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs, Registers *needed_registers, NamedArgument is_named, ParamInfo param);
 static bool x64_type_is_structure(Type *type);
 static void x64_classify(Type *type, ByteSize offset_base, X64Class *lo_class, X64Class *hi_class, NamedArgument named);
 
-ABIArgInfo *x64_indirect_return_result(Type *type)
+ABIArgInfo *x64_indirect_return_result(Type *type, ParamInfo param)
 {
 	if (type_is_abi_aggregate(type))
 	{
-		return abi_arg_new_indirect_not_by_val(type);
+		return abi_arg_new_indirect_not_by_val(type, param);
 	}
 	type = type_lowering(type);
 	if (type_is_promotable_int_bool(type))
 	{
-		return abi_arg_new_direct_int_ext(type);
+		return abi_arg_new_direct_int_ext(type, param);
 	}
-	return abi_arg_new_direct();
+	return abi_arg_new_direct(param);
 }
 
 
@@ -72,7 +72,7 @@ static bool x64_type_is_illegal_vector(Type *type)
 	return false;
 }
 
-ABIArgInfo *x64_indirect_result(Type *type, unsigned free_int_regs)
+ABIArgInfo *x64_indirect_result(Type *type, unsigned free_int_regs, ParamInfo param)
 {
 	// If this is a scalar LLVM value then assume LLVM will pass it in the right
 	// place naturally.
@@ -87,10 +87,10 @@ ABIArgInfo *x64_indirect_result(Type *type, unsigned free_int_regs)
 	{
 		if (type_is_promotable_int_bool(type))
 		{
-			return abi_arg_new_direct_int_ext(type);
+			return abi_arg_new_direct_int_ext(type, param);
 		}
 		// No change, just put it on the stack
-		return abi_arg_new_direct();
+		return abi_arg_new_direct(param);
 	}
 
 	// The byval alignment
@@ -103,14 +103,14 @@ ABIArgInfo *x64_indirect_result(Type *type, unsigned free_int_regs)
 		ByteSize size = type_size(type);
 		if (align <= 8 && size <= 8)
 		{
-			return abi_arg_new_direct_coerce_int();
+			return abi_arg_new_direct_coerce_int(param);
 		}
 	}
 	if (align < 8)
 	{
-		return abi_arg_new_indirect_realigned(8, type);
+		return abi_arg_new_indirect_realigned(8, type, param);
 	}
-	return abi_arg_new_indirect_by_val(type);
+	return abi_arg_new_indirect_by_val(type, param);
 }
 
 
@@ -120,7 +120,7 @@ ABIArgInfo *x64_indirect_result(Type *type, unsigned free_int_regs)
  * @param needed_registers
  * @return
  */
-ABIArgInfo *x64_classify_reg_call_struct_type_check(Type *type, Registers *needed_registers)
+ABIArgInfo *x64_classify_reg_call_struct_type_check(Type *type, Registers *needed_registers, ParamInfo param)
 {
 	ASSERT(x64_type_is_structure(type));
 
@@ -128,40 +128,40 @@ ABIArgInfo *x64_classify_reg_call_struct_type_check(Type *type, Registers *neede
 	if (type->type_kind == TYPE_SLICE || type->type_kind == TYPE_ANY)
 	{
 		needed_registers->int_registers += 2;
-		return abi_arg_new_direct();
+		return abi_arg_new_direct(param);
 	}
 
 	// Struct, err type handled =>
 	ASSERT(type->type_kind == TYPE_STRUCT);
 
 	// Variable array structs are always passed by pointer.
-	if (type->decl->has_variable_array) return x64_indirect_return_result(type);
+	if (type->decl->has_variable_array) return x64_indirect_return_result(type, param);
 
 	FOREACH(Decl *, member, type->decl->strukt.members)
 	{
-		Type *member_type = type_lowering(member->type->canonical);
+		Type *member_type = lowered_member_type(member);
 		ABIArgInfo *member_info;
 		Registers temp_needed_registers = { 0, 0 };
 		if (x64_type_is_structure(member_type))
 		{
 			// Recursively check the structure.
-			member_info = x64_classify_reg_call_struct_type_check(member_type, &temp_needed_registers);
+			member_info = x64_classify_reg_call_struct_type_check(member_type, &temp_needed_registers, param);
 		}
 		else
 		{
 			// Pass as single argument.
-			member_info = x64_classify_argument_type(member_type, ~(0U), &temp_needed_registers, NAMED);
+			member_info = x64_classify_argument_type(member_type, ~(0U), &temp_needed_registers, NAMED, param);
 		}
 		if (abi_arg_is_indirect(member_info))
 		{
 			*needed_registers = (Registers) { 0, 0 };
-			return x64_indirect_return_result(type);
+			return x64_indirect_return_result(type, param);
 		}
 		needed_registers->sse_registers += temp_needed_registers.sse_registers;
 		needed_registers->int_registers += temp_needed_registers.int_registers;
 	}
 	// Send as direct.
-	return abi_arg_new_direct();
+	return abi_arg_new_direct(param);
 }
 
 
@@ -232,8 +232,9 @@ void x64_classify_struct_union(Type *type, ByteSize offset_base, X64Class *curre
 		// The only case a 256-bit or a 512-bit wide vector could be used is when
 		// the struct contains a single 256-bit or 512-bit field. Early check
 		// and fallback to memory.
+		Type *member_type = lowered_member_type(member);
 		if (size > 16 &&
-			((!is_union && size != type_size(member->type))
+			((!is_union && size != type_size(member_type))
 			|| size > compiler.platform.x64.native_vector_size_avx))
 		{
 			*lo_class = CLASS_MEMORY;
@@ -241,7 +242,7 @@ void x64_classify_struct_union(Type *type, ByteSize offset_base, X64Class *curre
 			return;
 		}
 		// Not aligned?
-		if (offset % type_abi_alignment(member->type))
+		if (offset % type_abi_alignment(member_type))
 		{
 			*lo_class = CLASS_MEMORY;
 			x64_classify_post_merge(size, lo_class, hi_class);
@@ -250,7 +251,7 @@ void x64_classify_struct_union(Type *type, ByteSize offset_base, X64Class *curre
 
 		X64Class field_lo;
 		X64Class field_hi;
-		x64_classify(member->type, offset, &field_lo, &field_hi, named_arg);
+		x64_classify(member_type, offset, &field_lo, &field_hi, named_arg);
 		*lo_class = x64_merge(*lo_class, field_lo);
 		*hi_class = x64_merge(*hi_class, field_hi);
 		if (*lo_class == CLASS_MEMORY || *hi_class == CLASS_MEMORY) break;
@@ -262,7 +263,7 @@ void x64_classify_struct_union(Type *type, ByteSize offset_base, X64Class *curre
 void x64_classify_array(Type *type, ByteSize offset_base, X64Class *current, X64Class *lo_class, X64Class *hi_class, NamedArgument named_arg)
 {
 	ByteSize size = type_size(type);
-	Type *element = type->array.base;
+	Type *element = type_lowering(type->array.base);
 	ByteSize element_size = type_size(element);
 	// Bigger than 64 bytes => MEM
 	if (size > 64) return;
@@ -361,11 +362,11 @@ static Decl *x64_get_member_at_offset(Decl *decl, unsigned offset)
 	return last_match;
 }
 
-static void x64_classify(Type *type, ByteSize offset_base, X64Class *lo_class, X64Class *hi_class, NamedArgument named)
+static void x64_classify(Type *type, ByteSize offset_base, X64Class *lo_class, X64Class *hi_class, NamedArgument named) // NOLINT
 {
 	*lo_class = CLASS_NO_CLASS;
 	*hi_class = CLASS_NO_CLASS;
-	X64Class *current = offset_base < 8 ? lo_class : hi_class;
+	X64Class *current = offset_base < 8 ? lo_class : hi_class; // NOLINT - avoid incorrect analysis
 	*current = CLASS_MEMORY;
 	type = type_lowering(type);
 	switch (type->type_kind)
@@ -432,14 +433,15 @@ bool x64_bits_contain_no_user_data(Type *type, unsigned start, unsigned end)
 	if (type->type_kind == TYPE_ARRAY)
 	{
 		// Check each field to see if the field overlaps with the queried range.
-		TypeSize element_size = type_size(type->array.base);
+		Type *element_type = lowered_array_element_type(type);
+		TypeSize element_size = type_size(element_type);
 		for (unsigned i = 0; i < type->array.len; i++)
 		{
 			// If the field is after the span we care about, then we're done..
 			TypeSize offset = i * element_size;
 			if (offset >= end) break;
 			unsigned element_start = offset < start ? start - offset : 0;
-			if (!x64_bits_contain_no_user_data(type->array.base, element_start, end - offset)) return false;
+			if (!x64_bits_contain_no_user_data(element_type, element_start, end - offset)) return false;
 		}
 		// No overlap
 		return true;
@@ -451,7 +453,7 @@ bool x64_bits_contain_no_user_data(Type *type, unsigned start, unsigned end)
 			unsigned offset = member->offset;
 			if (offset >= end) break;
 			unsigned field_start = offset < start ? start - offset : 0;
-			if (!x64_bits_contain_no_user_data(member->type, field_start, end - offset)) return false;
+			if (!x64_bits_contain_no_user_data(lowered_member_type(member), field_start, end - offset)) return false;
 		}
 		// No overlap
 		return true;
@@ -459,7 +461,7 @@ bool x64_bits_contain_no_user_data(Type *type, unsigned start, unsigned end)
 	return false;
 }
 
-bool x64_contains_float_at_offset(Type *type, unsigned offset)
+bool x64_contains_float_at_offset(LoweredType *type, unsigned offset)
 {
 	if (offset == 0 && type->type_kind == TYPE_F32) return true;
 
@@ -468,11 +470,11 @@ bool x64_contains_float_at_offset(Type *type, unsigned offset)
 	{
 		Decl *member = x64_get_member_at_offset(type->decl, offset);
 		offset -= member->offset;
-		return x64_contains_float_at_offset(member->type, offset);
+		return x64_contains_float_at_offset(lowered_member_type(member), offset);
 	}
 	if (type->type_kind == TYPE_ARRAY)
 	{
-		Type *element_type = type->array.base;
+		Type *element_type = lowered_array_element_type(type);
 		unsigned element_size = type_size(element_type);
 		offset -= (offset / element_size) * element_size;
 		return x64_contains_float_at_offset(element_type, offset);
@@ -490,17 +492,17 @@ static Type *x64_get_fp_type_at_offset(Type *type, unsigned ir_offset)
 	}
 	if (type->type_kind == TYPE_ARRAY)
 	{
-		Type *element_type = type_lowering(type->array.base);
+		Type *element_type = lowered_array_element_type(type);
 		ByteSize size = type_size(element_type);
 		return x64_get_fp_type_at_offset(element_type, ir_offset - size * (ir_offset / size));
 	}
 	return NULL;
 }
 
-static Type *x64_get_sse_type_at_offset(Type *type, unsigned ir_offset, Type *source_type, unsigned source_offset)
+static AbiType x64_get_sse_type_at_offset(Type *type, unsigned ir_offset, Type *source_type, unsigned source_offset)
 {
 	Type *float_type = x64_get_fp_type_at_offset(type, ir_offset);
-	if (!float_type || float_type == type_double) return type_double;
+	if (!float_type || float_type == type_double) return abi_type_get(type_double);
 
 	ByteSize size = type_size(float_type);
 	ByteSize source_size = type_size(source_type) - source_offset;
@@ -520,25 +522,30 @@ static Type *x64_get_sse_type_at_offset(Type *type, unsigned ir_offset, Type *so
 			float_type2 = x64_get_fp_type_at_offset(type, ir_offset + 4);
 		}
 		// If we can't get a second FP type, return a simple half or float.
-		if (!float_type2) return float_type;
+		if (!float_type2) return abi_type_get(float_type);
 	}
 
 	if (float_type == type_float && float_type == float_type2)
 	{
-		return type_get_vector(float_type, 2);
+		return abi_type_spec(ABI_TYPE_FLOAT_VEC_2);
 	}
 
 	if (type_is_16bit_float(float_type) && type_is_16bit_float(float_type2))
 	{
 		bool has_following_float = source_size > 4
 				&& x64_get_fp_type_at_offset(type, ir_offset + 4) != NULL;
-		return type_get_vector(float_type, has_following_float ? 4 : 2);
+		if (float_type->type_kind == TYPE_BF16)
+		{
+			return abi_type_spec(has_following_float ? ABI_TYPE_BFLOAT16_VEC_4 : ABI_TYPE_BFLOAT16_VEC_2);
+		}
+		return abi_type_spec(has_following_float ? ABI_TYPE_FLOAT16_VEC_4 : ABI_TYPE_FLOAT16_VEC_2);
 	}
+
 	if (type_is_16bit_float(float_type) || type_is_16bit_float(float_type2))
 	{
-		return type_get_vector(type_float16, 4);
+		return abi_type_spec(ABI_TYPE_FLOAT16_VEC_4);
 	}
-	return type_double;
+	return abi_type_get(type_double);
 }
 
 /**
@@ -579,7 +586,7 @@ AbiType x64_get_int_type_at_offset(Type *type, unsigned offset, Type *source_typ
 			Decl *member = x64_get_member_at_offset(type->decl, offset);
 			if (member)
 			{
-				return x64_get_int_type_at_offset(member->type, offset - member->offset, source_type, source_offset);
+				return x64_get_int_type_at_offset(lowered_member_type(member), offset - member->offset, source_type, source_offset);
 			}
 			break;
 		}
@@ -595,7 +602,7 @@ AbiType x64_get_int_type_at_offset(Type *type, unsigned offset, Type *source_typ
 			UNREACHABLE_VOID
 		case TYPE_ARRAY:
 		{
-			Type *element = type->array.base;
+			Type *element = lowered_array_element_type(type);
 			TypeSize element_size = type_size(element);
 			TypeSize element_offset = (offset / element_size) * element_size;
 			return x64_get_int_type_at_offset(element, offset - element_offset, source_type, source_offset);
@@ -621,7 +628,7 @@ static AbiType x64_get_byte_vector_type(Type *type)
 {
 	// Wrapper structs/arrays that only contain vectors are passed just like
 	// vectors; strip them off if present.
-	Type *inner_type = type_abi_find_single_struct_element(type);
+	Type *inner_type = type_abi_find_single_struct_element(type, true);
 	if (inner_type) type = inner_type;
 	type = type_lowering(type);
 
@@ -632,7 +639,7 @@ static AbiType x64_get_byte_vector_type(Type *type)
 		if (compiler.platform.x64.pass_int128_vector_in_mem && type_is_int128(element))
 		{
 			// Convert to u64
-			return abi_type_get(type_get_vector(type_ulong, type_size(type) / 8));
+			return (AbiType) { .abi_type = ABI_TYPE_LONG_VEC_2 };
 		}
 		return abi_type_get(type);
 	}
@@ -644,20 +651,29 @@ static AbiType x64_get_byte_vector_type(Type *type)
 	ASSERT(size == 16 || size == 32 || size == 64);
 
 	// Return a vector type based on the size.
-	return abi_type_get(type_get_vector(type_double, size / 8));
+	switch (size)
+	{
+		case 16: return (AbiType) { .abi_type = ABI_TYPE_DOUBLE_VEC_2 };
+		case 32: return (AbiType) { .abi_type = ABI_TYPE_DOUBLE_VEC_4 };
+		case 64: return (AbiType) { .abi_type = ABI_TYPE_DOUBLE_VEC_8 };
+		default:
+			UNREACHABLE_VOID;
+			return (AbiType) { .type = NULL };
+	}
 }
 
-static ABIArgInfo *x64_get_argument_pair_return(AbiType low_type, AbiType high_type)
+static ABIArgInfo *x64_get_argument_pair_return(AbiType low_type, AbiType high_type, ParamInfo param)
 {
 	TypeSize low_size = abi_type_size(low_type);
 	unsigned hi_start = aligned_offset(low_size, abi_type_abi_alignment(high_type));
 	ASSERT(hi_start == 8 && "Expected aligned with C-style structs.");
-	return abi_arg_new_direct_pair(low_type, high_type);
+	return abi_arg_new_direct_pair(low_type, high_type, param);
 }
 
 
-ABIArgInfo *x64_classify_return(Type *return_type)
+ABIArgInfo *x64_classify_return(ParamInfo param)
 {
+	Type *return_type = type_lowering(param.type);
 	// AMD64-ABI 3.2.3p4: Rule 1. Classify the return type with the
 	// classification algorithm.
 	X64Class hi_class;
@@ -684,18 +700,18 @@ ABIArgInfo *x64_classify_return(Type *return_type)
 		case CLASS_MEMORY:
 			// AMD64-ABI 3.2.3p4: Rule 2. Types of class memory are returned via
 			// hidden argument.
-			return x64_indirect_return_result(return_type);
+			return x64_indirect_return_result(return_type, param);
 		case CLASS_INTEGER:
 			// AMD64-ABI 3.2.3p4: Rule 3. If the class is INTEGER, the next
 			// available register of the sequence %rax, %rdx is used.
 			result_type = x64_get_int_type_at_offset(return_type, 0, return_type, 0);
 			if (hi_class == CLASS_NO_CLASS && type_is_promotable_int_bool(return_type))
 			{
-				return abi_arg_new_direct_coerce_int_ext(return_type);
+				return abi_arg_new_direct_coerce_int_ext(return_type, param);
 			}
 			break;
 		case CLASS_SSE:
-			result_type = abi_type_get(x64_get_sse_type_at_offset(return_type, 0, return_type, 0));
+			result_type = x64_get_sse_type_at_offset(return_type, 0, return_type, 0);
 			break;
 		default:
 			UNREACHABLE
@@ -714,7 +730,7 @@ ABIArgInfo *x64_classify_return(Type *return_type)
 			break;
 		case CLASS_SSE:
 			ASSERT(lo_class != CLASS_NO_CLASS);
-			high_part = abi_type_get(x64_get_sse_type_at_offset(return_type, 8, return_type, 8));
+			high_part = x64_get_sse_type_at_offset(return_type, 8, return_type, 8);
 			break;
 		case CLASS_SSEUP:
 			// AMD64-ABI 3.2.3p4: Rule 5. If the class is SSEUP, the eightbyte
@@ -730,18 +746,13 @@ ABIArgInfo *x64_classify_return(Type *return_type)
 	// If a high part was specified, merge it together with the low part.  It is
 	// known to pass in the high eightbyte of the result.  We do this by forming a
 	// first class struct aggregate with the high and low part: {low, high}
-	if (abi_type_is_valid(high_part)) return x64_get_argument_pair_return(result_type, high_part);
+	if (abi_type_is_valid(high_part)) return x64_get_argument_pair_return(result_type, high_part, param);
 
-	if (abi_type_is_type(result_type))
+	if (abi_type_match(result_type, return_type))
 	{
-		if (return_type->canonical == result_type.type->canonical)
-		{
-			return abi_arg_new_direct();
-		}
-		return abi_arg_new_direct_coerce_type(result_type.type->canonical);
+		return abi_arg_new_direct(param);
 	}
-	ASSERT(result_type.int_bits_plus_1 - 1 == type_size(return_type) * 8);
-	return abi_arg_new_direct_coerce_int();
+	return abi_arg_new_direct_coerce_type(result_type, param);
 }
 
 /**
@@ -754,7 +765,7 @@ ABIArgInfo *x64_classify_return(Type *return_type)
  * @param is_named
  * @return
  */
-static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs, Registers *needed_registers, NamedArgument is_named)
+static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs, Registers *needed_registers, NamedArgument is_named, ParamInfo param)
 {
 	ASSERT(type == type_lowering(type));
 	X64Class hi_class;
@@ -765,7 +776,7 @@ static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs
 	ASSERT(hi_class != CLASS_MEMORY || lo_class == CLASS_MEMORY);
 	ASSERT(hi_class != CLASS_SSEUP || lo_class == CLASS_SSE);
 
-	AbiType result_type;
+	AbiType result_type = ABI_TYPE_EMPTY;
 	*needed_registers = (Registers) { 0, 0 };
 
 	// Start by checking the lower class.
@@ -778,18 +789,18 @@ static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs
 		case CLASS_SSEUP:
 			UNREACHABLE
 		case CLASS_MEMORY:
-			return x64_indirect_result(type, free_int_regs);
+			return x64_indirect_result(type, free_int_regs, param);
 		case CLASS_INTEGER:
 			needed_registers->int_registers++;
 			result_type = x64_get_int_type_at_offset(type, 0, type, 0);
 			if (hi_class == CLASS_NO_CLASS && type_is_promotable_int_bool(type))
 			{
 				ASSERT(abi_type_is_type(result_type));
-				return abi_arg_new_direct_coerce_int_ext(result_type.type);
+				return abi_arg_new_direct_coerce_int_ext(result_type.type, param);
 			}
 			break;
 		case CLASS_SSE:
-			result_type = abi_type_get(x64_get_sse_type_at_offset(type, 0, type, 0));
+			result_type = x64_get_sse_type_at_offset(type, 0, type, 0);
 			needed_registers->sse_registers++;
 			break;
 	}
@@ -810,7 +821,7 @@ static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs
 			break;
 		case CLASS_SSE:
 			needed_registers->sse_registers++;
-			high_part = abi_type_get(x64_get_sse_type_at_offset(type, 8, type, 8));
+			high_part = x64_get_sse_type_at_offset(type, 8, type, 8);
 			ASSERT(lo_class != CLASS_NO_CLASS && "empty first 8 bytes not allowed, this is C++ stuff");
 			break;
 		case CLASS_SSEUP:
@@ -822,21 +833,21 @@ static ABIArgInfo *x64_classify_argument_type(Type *type, unsigned free_int_regs
 	// If a high part was specified, merge it together with the low part.  It is
 	// known to pass in the high eightbyte of the result.  We do this by forming a
 	// first class struct aggregate with the high and low part: {low, high}
-	if (abi_type_is_valid(high_part)) return x64_get_argument_pair_return(result_type, high_part);
+	if (abi_type_is_valid(high_part)) return x64_get_argument_pair_return(result_type, high_part, param);
 
+	if (abi_type_match(result_type, type))
+	{
+		return abi_arg_new_direct(param);
+	}
 	if (abi_type_is_type(result_type))
 	{
 		Type *result = result_type.type->canonical;
-		type = type->canonical;
-		if (type == result) return abi_arg_new_direct();
 		if (type_is_integer(type) && type_is_integer(result) && type->builtin.bytesize == result->builtin.bytesize)
 		{
-			return abi_arg_new_direct();
+			return abi_arg_new_direct(param);
 		}
-		return abi_arg_new_direct_coerce_type(result);
 	}
-	ASSERT(result_type.int_bits_plus_1 - 1 == type_size(type) * 8);
-	return abi_arg_new_direct_coerce_int();
+	return abi_arg_new_direct_coerce_type(result_type, param);
 }
 
 bool x64_type_is_structure(Type *type)
@@ -854,31 +865,30 @@ bool x64_type_is_structure(Type *type)
 
 /**
  * This code is based on the loop operations in X86_64ABIInfo::computeInfo in Clang
- * @param type
+ * @param param param info
  * @param available_registers to update
- * @param is_regcall true if this is a regcall
  * @param named whether this is a named (non-vararg) parameter or not.
  * @return the calculated ABI
  */
-static ABIArgInfo *x64_classify_parameter(Type *type, Registers *available_registers, NamedArgument named)
+static ABIArgInfo *x64_classify_parameter(ParamInfo param, Registers *available_registers, NamedArgument named)
 {
 	Registers needed_registers = { 0, 0 };
-	type = type_lowering(type);
-	ABIArgInfo *info = x64_classify_argument_type(type, available_registers->int_registers, &needed_registers, named);
+	Type *type = type_lowering(param.type);
+	ABIArgInfo *info = x64_classify_argument_type(type, available_registers->int_registers, &needed_registers, named, param);
 
 	// Check if we can fit in a register, we're golden.
 	if (try_use_registers(available_registers, &needed_registers)) return info;
 
 	// The rest needs to be passed indirectly.
-	return x64_indirect_result(type, available_registers->int_registers);
+	return x64_indirect_result(type, available_registers->int_registers, param);
 
 }
 
-void c_abi_func_create_x64(FunctionPrototype *prototype)
+void c_abi_func_create_x64(FunctionPrototype *prototype, ParamInfo *params, unsigned param_count, ParamInfo *vaargs, unsigned vaarg_count)
 {
 	if (prototype->use_win64)
 	{
-		c_abi_func_create_win64(prototype);
+		c_abi_func_create_win64(prototype, params, param_count, vaargs, vaarg_count);
 		return;
 	}
 
@@ -887,18 +897,9 @@ void c_abi_func_create_x64(FunctionPrototype *prototype)
 			.sse_registers = 8
 	};
 
-	prototype->ret_abi_info = x64_classify_return(type_lowering(prototype->abi_ret_type));
+	prototype->ret_abi_info = x64_classify_return(prototype->return_info);
 	if (abi_arg_is_indirect(prototype->ret_abi_info)) available_registers.int_registers--;
 
-	if (prototype->ret_by_ref)
-	{
-		prototype->ret_by_ref_abi_info = x64_classify_parameter(type_get_ptr(type_lowering(prototype->ret_by_ref_type)),
-																&available_registers,
-																NAMED);
-	}
-
-	Type **params = prototype->param_types;
-	unsigned param_count = vec_size(prototype->param_types);
 	if (param_count)
 	{
 		ABIArgInfo **args = MALLOC(sizeof(ABIArgInfo) * param_count);
@@ -908,13 +909,12 @@ void c_abi_func_create_x64(FunctionPrototype *prototype)
 		}
 		prototype->abi_args = args;
 	}
-	unsigned vararg_count = vec_size(prototype->varargs);
-	if (vararg_count)
+	if (vaarg_count)
 	{
-		ABIArgInfo **args = MALLOC(sizeof(ABIArgInfo) * vararg_count);
-		for (unsigned i = 0; i < vararg_count; i++)
+		ABIArgInfo **args = MALLOC(sizeof(ABIArgInfo) * vaarg_count);
+		for (unsigned i = 0; i < vaarg_count; i++)
 		{
-			args[i] = x64_classify_parameter(prototype->varargs[i], &available_registers, UNNAMED);
+			args[i] = x64_classify_parameter(vaargs[i], &available_registers, UNNAMED);
 		}
 		prototype->abi_varargs = args;
 	}
