@@ -116,7 +116,7 @@ static void compiler_lex(void)
 		bool loaded = false;
 		const char *error;
 		File *file = source_file_load(source, &loaded, &error);
-		if (!file) error_exit(error);
+		if (!file) error_exit("%s", error);
 		if (loaded) continue;
 		Lexer lexer = { .file = file };
 		lexer_init(&lexer);
@@ -401,7 +401,7 @@ void compiler_parse(void)
 		bool loaded = false;
 		const char *error;
 		File *file = source_file_load(source, &loaded, &error);
-		if (!file) error_exit(error);
+		if (!file) error_exit("%s", error);
 		if (loaded) continue;
 		if (!parse_file(file)) has_error = true;
 		if (compiler.build.print_input) puts(file->full_path);
@@ -1171,8 +1171,8 @@ void print_syntax(BuildOptions *options)
 		puts(" 8. Relational | < > <= >= == !=");
 		puts(" 9. And        | && &&&");
 		puts("10. Or         | || |||");
-		puts("11. Ternary    | ?: ??");
-		puts("12. Assign     | = *= /= %= -= += |= &= ^= <<= >>=");
+		puts("11. Ternary    | ?: ?? ???");
+		puts("12. Assign     | = *= /= %= -= += |= &= ^= <<= >>= +++=");
 	}
 
 }
