@@ -270,6 +270,7 @@ typedef enum
 	ATTRIBUTE_BIGENDIAN,
 	ATTRIBUTE_BUILTIN,
 	ATTRIBUTE_CALLCONV,
+	ATTRIBUTE_CNAME,
 	ATTRIBUTE_COMPACT,
 	ATTRIBUTE_CONST,
 	ATTRIBUTE_DEPRECATED,
@@ -309,6 +310,7 @@ typedef enum
 	ATTRIBUTE_REFLECT,
 	ATTRIBUTE_SAFEINFER,
 	ATTRIBUTE_SAFEMACRO,
+	ATTRIBUTE_SIMD,
 	ATTRIBUTE_SECTION,
 	ATTRIBUTE_STRUCTLIKE,
 	ATTRIBUTE_TAG,
@@ -507,10 +509,13 @@ typedef enum
 	BUILTIN_SELECT,
 	BUILTIN_SET_ROUNDING_MODE,
 	BUILTIN_SPRINTF,
-	BUILTIN_STR_HASH,
-	BUILTIN_STR_UPPER,
-	BUILTIN_STR_LOWER,
 	BUILTIN_STR_FIND,
+	BUILTIN_STR_HASH,
+	BUILTIN_STR_LOWER,
+	BUILTIN_STR_PASCALCASE,
+	BUILTIN_STR_REPLACE,
+	BUILTIN_STR_UPPER,
+	BUILTIN_STR_SNAKECASE,
 	BUILTIN_SWIZZLE,
 	BUILTIN_SWIZZLE2,
 	BUILTIN_SIN,
@@ -787,6 +792,7 @@ typedef enum
 	EXPR_MACRO_BODY_EXPANSION,
 	EXPR_MAKE_ANY,
 	EXPR_MAKE_SLICE,
+	EXPR_MAYBE_DEREF,
 	EXPR_MEMBER_GET,
 	EXPR_MEMBER_SET,
 	EXPR_NAMED_ARGUMENT,
@@ -1616,6 +1622,7 @@ typedef enum
 	X86_FEAT_RTM,
 	X86_FEAT_SAHF,
 	X86_FEAT_SERIALIZE,
+	X86_FEAT_SOFT_FLOAT,
 	X86_FEAT_SGX,
 	X86_FEAT_SHA,
 	X86_FEAT_SHA512,
@@ -1648,6 +1655,102 @@ typedef enum
 	X86_FEATURE_LAST = X86_FEAT_XSAVES,
 } X86Feature;
 
+typedef enum
+{
+	WASM_FEAT_BULK_MEMORY,
+	WASM_FEAT_BULK_MEMORY_OPT,
+	WASM_FEAT_CALL_INDIRECT_OVERLONG,
+	WASM_FEAT_MULTIVALUE,
+	WASM_FEAT_MUTABLE_GLOBALS,
+	WASM_FEAT_NONTRAPPING_FPTORINT,
+	WASM_FEAT_REFERENCE_TYPES,
+	WASM_FEAT_SIGN_EXT,
+	WASM_FEATURE_LAST = WASM_FEAT_SIGN_EXT,
+} WasmFeature;
+
+typedef enum
+{
+	ARM_FEAT_ARMV6KZ,
+	ARM_FEAT_DSP,
+	ARM_FEAT_FP64,
+	ARM_FEAT_STRICT_ALIGN,
+	ARM_FEAT_VFP2,
+	ARM_FEAT_VFP2SP,
+	ARM_FEAT_AES,
+	ARM_FEAT_D32,
+	ARM_FEAT_FP_ARMV8,
+	ARM_FEAT_FP_ARMV8D16,
+	ARM_FEAT_FP_ARMV8D16SP,
+	ARM_FEAT_FP_ARMV8SP,
+	ARM_FEAT_FP_FP16,
+	ARM_FEAT_FP_FP16FML,
+	ARM_FEAT_FP_FULLFP16,
+	ARM_FEAT_NEON,
+	ARM_FEAT_SHA2,
+	ARM_FEAT_THUMB_MODE,
+	ARM_FEAT_VFP3,
+	ARM_FEAT_VFP3D16,
+	ARM_FEAT_VFP3D16SP,
+	ARM_FEAT_VFP3SP,
+	ARM_FEAT_VFP4,
+	ARM_FEAT_VFP4D16,
+	ARM_FEAT_VFP4D16SP,
+	ARM_FEAT_VFP4SP,
+	ARM_FEAT_LAST = ARM_FEAT_VFP4SP,
+} ArmFeature;
+
+typedef enum
+{
+	AARCH64_FEAT_CRC,
+	AARCH64_FEAT_LSE,
+	AARCH64_FEAT_RDM,
+	AARCH64_FEAT_FP_ARMV8,
+	AARCH64_FEAT_NEON,
+	AARCH64_FEATURE_LAST = AARCH64_FEAT_NEON,
+} Aarch64Feature;
+
+typedef enum RiscVFeature
+{
+	RISCV_FEAT_64BIT,
+	RISCV_FEAT_32BIT,
+	RISCV_FEAT_A,
+	RISCV_FEAT_B,
+	RISCV_FEAT_C,
+	RISCV_FEAT_D,
+	RISCV_FEAT_E,
+	RISCV_FEAT_F,
+	RISCV_FEAT_I,
+	RISCV_FEAT_M,
+	RISCV_FEAT_RELAX,
+	RISCV_FEAT_V,
+	RISCV_FEAT_ZAAMO,
+	RISCV_FEAT_ZALRSC,
+	RISCV_FEAT_ZBA,
+	RISCV_FEAT_ZBB,
+	RISCV_FEAT_ZBS,
+	RISCV_FEAT_ZCA,
+	RISCV_FEAT_ZCD,
+	RISCV_FEAT_ZCF,
+	RISCV_FEAT_ZFA,
+	RISCV_FEAT_ZICSR,
+	RISCV_FEAT_ZIFENCEI,
+	RISCV_FEAT_ZMMUL,
+	RISCV_FEATURE_LAST = RISCV_FEAT_ZMMUL,
+} RiscVFeature;
+
+typedef enum
+{
+	RET_NORMAL,
+	RET_OPTIONAL_VOID,
+	RET_OPTIONAL_VALUE,
+} RetValType;
+typedef enum
+{
+	PARAM_RW_NONE,
+	PARAM_RW_VEC_TO_ARRAY,
+	PARAM_RW_EXPAND_ELEMENTS,
+} ParamRewrite;
+
 // -- Arch helper macros
 #define ARCH_UNSUPPORTED ARCH_TYPE_AARCH64_32: case ARCH_TYPE_BPFEL: case ARCH_TYPE_BPFEB: case ARCH_TYPE_SPARCEL: \
  case ARCH_TYPE_LE64: case ARCH_TYPE_AMDIL: case ARCH_TYPE_AMDIL64: case ARCH_TYPE_HSAIL: case ARCH_TYPE_HSAIL64:  \
@@ -1679,7 +1782,7 @@ typedef enum
 		case EXPR_CT_ARG: case EXPR_TYPEINFO: case EXPR_CT_IDENT: case EXPR_HASH_IDENT: \
 		case EXPR_COMPILER_CONST: case EXPR_CT_CALL:  \
 		case EXPR_SPLAT: case EXPR_STRINGIFY: case EXPR_TYPECALL: \
-		case EXPR_CT_EVAL
+		case EXPR_CT_EVAL: case EXPR_MAYBE_DEREF
 
 static_assert(EXPR_LAST < 128, "Too many expression types");
 
