@@ -355,6 +355,7 @@ bool expr_is_runtime_const(Expr *expr)
 		case EXPR_VECTOR_TO_ARRAY:
 		case EXPR_SLICE_TO_VEC_ARRAY:
 		case EXPR_SCALAR_TO_VECTOR:
+		case EXPR_MAYBE_DEREF:
 			return expr_is_runtime_const(expr->inner_expr);
 		case EXPR_MAKE_SLICE:
 			expr = expr->make_slice_expr.ptr;
@@ -420,7 +421,7 @@ bool expr_is_runtime_const(Expr *expr)
 		case EXPR_INITIALIZER_LIST:
 			return expr_list_is_constant_eval(expr->initializer_list);
 		case EXPR_DESIGNATED_INITIALIZER_LIST:
-			return expr_list_is_constant_eval(expr->designated_init_list);
+			return (!expr->designated_init.splat || expr_is_const(expr->designated_init.splat)) && expr_list_is_constant_eval(expr->designated_init.list);
 		case EXPR_SLICE:
 			if (!exprid_is_runtime_const(expr->slice_expr.expr)) return false;
 			return expr->slice_expr.range.range_type == RANGE_CONST_RANGE;
@@ -589,8 +590,7 @@ void expr_insert_addr(Expr *original)
 	Expr *inner = expr_copy(original);
 	original->expr_kind = EXPR_UNARY;
 	original->type = new_type;
-	original->unary_expr.operator = UNARYOP_ADDR;
-	original->unary_expr.expr = inner;
+	original->unary_expr = (ExprUnary) { .operator = UNARYOP_ADDR, .expr = inner };
 }
 
 Expr *expr_generated_local(Expr *assign, Decl **decl_ref)
@@ -819,6 +819,7 @@ bool expr_is_pure(Expr *expr)
 		case EXPR_RVALUE:
 		case EXPR_RECAST:
 		case EXPR_ADDR_CONVERSION:
+		case EXPR_MAYBE_DEREF:
 			return expr_is_pure(expr->inner_expr);
 		case EXPR_INT_TO_BOOL:
 			return expr_is_pure(expr->int_to_bool_expr.inner);
