@@ -1101,6 +1101,9 @@ void llvm_emit_builtin_call(GenContext *c, BEValue *result_value, Expr *expr)
 		case BUILTIN_STR_LOWER:
 		case BUILTIN_STR_UPPER:
 		case BUILTIN_STR_FIND:
+		case BUILTIN_STR_REPLACE:
+		case BUILTIN_STR_SNAKECASE:
+		case BUILTIN_STR_PASCALCASE:
 		case BUILTIN_WIDESTRING_16:
 		case BUILTIN_WIDESTRING_32:
 		case BUILTIN_RND:
