@@ -124,8 +124,7 @@ INLINE LLVMValueRef llvm_store_to_ptr_raw(GenContext *c, LLVMValueRef pointer, L
 INLINE void llvm_value_bitcast(GenContext *c UNUSED, BEValue *value, Type *type)
 {
 	ASSERT(llvm_value_is_addr(value));
-	type = type_lowering(type);
-	value->type = type;
+	value->type = type_lowering(type);
 }
 
 INLINE LLVMValueRef llvm_emit_shl(GenContext *c, LLVMValueRef value, LLVMValueRef shift)
@@ -155,7 +154,7 @@ INLINE LLVMValueRef llvm_emit_trunc_bool(GenContext *c, LLVMValueRef value)
 
 INLINE LLVMValueRef llvm_emit_extract_value(GenContext *c, LLVMValueRef agg, unsigned index)
 {
-	if (LLVMGetTypeKind(LLVMTypeOf(agg)) == LLVMVectorTypeKind)
+	if (LLVMGetTypeKind(LLVMTypeOf(agg)) == LLVMVectorTypeKind )
 	{
 		return LLVMBuildExtractElement(c->builder, agg, llvm_const_int(c, type_usz, index), "");
 	}
