@@ -862,13 +862,18 @@ static Expr *parse_initializer_list(ParseContext *c, Expr *left, SourceSpan lhs_
 	ASSERT(!left && "Unexpected left hand side");
 	Expr *initializer_list = EXPR_NEW_TOKEN(EXPR_INITIALIZER_LIST);
 	advance_and_verify(c, TOKEN_LBRACE);
+	Expr *splat = NULL;
 	if (!try_consume(c, TOKEN_RBRACE))
 	{
 		Expr **exprs = NULL;
 		if (!parse_init_list(c, &exprs, TOKEN_RBRACE, NULL, true)) return poisoned_expr;
 		int designated = -1;
-		FOREACH(Expr *, expr, exprs)
+		FOREACH_IDX(i, Expr *, expr, exprs)
 		{
+			if (i == 0 && expr->expr_kind == EXPR_SPLAT)
+			{
+				splat = expr;
+			}
 			if (expr->expr_kind == EXPR_DESIGNATOR)
 			{
 				if (designated == 0)
@@ -876,10 +881,12 @@ static Expr *parse_initializer_list(ParseContext *c, Expr *left, SourceSpan lhs_
 					designated = expr->designator_expr.path[0]->kind == DESIGNATOR_FIELD ? 1 : 2;
 					goto ERROR;
 				}
+
 				designated = expr->designator_expr.path[0]->kind == DESIGNATOR_FIELD ? 1 : 2;
 				continue;
 			}
 			if (designated > 0) goto ERROR;
+			if (designated == -1 && splat) continue;
 			designated = 0;
 			continue;
 ERROR:;
@@ -896,7 +903,12 @@ ERROR:;
 		RANGE_EXTEND_PREV(initializer_list);
 		if (designated > 0)
 		{
-			initializer_list->designated_init_list = exprs;
+			if (splat)
+			{
+				vec_erase_front(exprs, 1);
+				splat = splat->inner_expr;
+			}
+			initializer_list->designated_init = (ExprDesignatedInit) { .splat = splat, .list = exprs };
 			initializer_list->expr_kind = EXPR_DESIGNATED_INITIALIZER_LIST;
 		}
 		else
@@ -1096,11 +1108,18 @@ static Expr *parse_generic_expr(ParseContext *c, Expr *left, SourceSpan lhs_star
 
 /**
  * access_expr ::= '.' primary_expr
+ * deref_subscript ::= '.' '[' expr ']'
  */
 static Expr *parse_access_expr(ParseContext *c, Expr *left, SourceSpan lhs_start)
 {
 	ASSERT(left && expr_ok(left));
 	advance_and_verify(c, TOKEN_DOT);
+	if (tok_is(c, TOKEN_LBRACKET))
+	{
+		Expr *deref = expr_new(EXPR_MAYBE_DEREF, left->span);
+		deref->inner_expr = left;
+		return parse_subscript_expr(c, deref, lhs_start);
+	}
 	Expr *access_expr = expr_new(EXPR_ACCESS_UNRESOLVED, lhs_start);
 	access_expr->access_unresolved_expr.parent = left;
 	ASSIGN_EXPR_OR_RET(access_expr->access_unresolved_expr.child, parse_precedence(c, PREC_PRIMARY), poisoned_expr);
