@@ -18,7 +18,7 @@ ArrayIndex sema_len_from_const(Expr *expr)
 			return range_const_len(&expr->slice_expr.range);
 		}
 		if (expr->expr_kind != EXPR_MAKE_SLICE) return -1;
-		return expr->make_slice_expr.len;
+		return (ArrayIndex)expr->make_slice_expr.len;
 	}
 	ConstInitializer *init;
 	switch (expr->const_expr.const_kind)
@@ -36,7 +36,7 @@ ArrayIndex sema_len_from_const(Expr *expr)
 			return -1;
 		case CONST_BYTES:
 		case CONST_STRING:
-			return expr->const_expr.bytes.len;
+			return (ArrayIndex)expr->const_expr.bytes.len;
 		case CONST_SLICE:
 			if (!expr->const_expr.slice_init) return 0;
 			init = expr->const_expr.slice_init;
@@ -45,11 +45,11 @@ ArrayIndex sema_len_from_const(Expr *expr)
 			init = expr->const_expr.initializer;
 			goto ARRAY_LEN;
 		case CONST_UNTYPED_LIST:
-			return vec_size(expr->const_expr.untyped_list);
+			return (ArrayIndex)vec_size(expr->const_expr.untyped_list);
 	}
 	UNREACHABLE
 ARRAY_LEN:
-	if (type_is_arraylike(init->type)) return init->type->array.len;
+	if (type_is_arraylike(init->type)) return (ArrayIndex)init->type->array.len;
 	return -1;
 }
 
@@ -203,7 +203,7 @@ static bool sema_append_concat_const_bytes(SemaContext *context, Expr *expr, Exp
 	char *current = data;
 	if (str_len) memcpy(current, list->const_expr.bytes.ptr, str_len);
 	current += str_len;
-	current[0] = (unsigned char)element->const_expr.ixx.i.low;
+	current[0] = (char)element->const_expr.ixx.i.low;
 	current[1] = '\0';
 	expr->expr_kind = EXPR_CONST;
 	expr->const_expr = (ExprConst) {
@@ -291,26 +291,24 @@ static inline ConstInitializer *expr_const_initializer_from_expr(Expr *expr)
 	}
 }
 
+
 /**
  * The following valid cases exist:
  *
  * 1. String/Bytes + ... => String/Bytes
  * 2. Vector/slice/array + Untyped list => Merged untyped list
- * 3. Vector/slice/array + arraylike => vector/array iff canoncial match, otherwise Untyped list
+ * 3. Vector/slice/array + arraylike => vector/array iff canonical match, otherwise Untyped list
  * 4. Untyped list + Vector/array/slice => Merged untyped list
  * 5. Vector/array/slice + element => Vector/array/slice + 1 len iff canonical match, Untyped list otherwise
  * 6. Untyped list + element => Untyped list
  */
-bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *left, Expr *right)
+bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *left, Expr *right, bool *failed_ref)
 {
-	ASSERT(concat_expr->resolve_status == RESOLVE_RUNNING);
+	ASSERT_SPAN(concat_expr, concat_expr->resolve_status == RESOLVE_RUNNING);
+	if (!sema_check_left_right_const(context, left, right)) return false;
 	ArraySize len = 0;
 	bool use_array = true;
 	Type *indexed_type = NULL;
-	if (!sema_analyse_expr(context, left)) return false;
-	if (!sema_cast_const(left)) RETURN_SEMA_ERROR(left, "Expected this to evaluate to a constant value.");
-	if (!sema_analyse_expr(context, right)) return false;
-	if (!sema_cast_const(right)) RETURN_SEMA_ERROR(right, "Expected this to evaluate to a constant value.");
 	Type *element_type = left->type->canonical;
 	Type *right_type = right->type->canonical;
 	switch (left->const_expr.const_kind)
@@ -324,6 +322,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 		case CONST_TYPEID:
 		case CONST_REF:
 		case CONST_MEMBER:
+			CHECK_ON_DEFINED(failed_ref);
 			RETURN_SEMA_ERROR(left, "Only bytes, strings and list-like constants can be concatenated.");
 		case CONST_BYTES:
 		case CONST_STRING:
@@ -343,6 +342,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 					break;
 				case TYPE_STRUCT:
 				case TYPE_UNION:
+					CHECK_ON_DEFINED(failed_ref);
 					RETURN_SEMA_ERROR(left, "Only bytes, strings and array-like constants can be concatenated.");
 				default:
 					UNREACHABLE
@@ -372,6 +372,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 			if (left->type == type_untypedlist || indexed_type == right_type) return sema_expr_const_append(context, concat_expr, left, right);
 			if (!type_is_integer(indexed_type) || type_size(indexed_type) != 1)
 			{
+				CHECK_ON_DEFINED(failed_ref);
 				RETURN_SEMA_ERROR(right, "You can't concatenate %s and %s.", type_quoted_error_string(left->type),
 				                  type_to_error_string(right_type));
 			}
@@ -483,6 +484,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 				.const_kind = CONST_UNTYPED_LIST,
 				.untyped_list = untyped_exprs
 		};
+
 		return true;
 	}
 	// Zero slice + zero slice => slice
@@ -540,7 +542,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 					FOREACH(ConstInitializer *, element, rhs_init->init_array.elements)
 					{
 						ASSERT_SPAN(right, element->kind == CONST_INIT_ARRAY_VALUE);
-						element->init_array_value.index += len_lhs;
+						element->init_array_value.index += (ArrayIndex)len_lhs;
 					}
 					rhs_init->type = type;
 					expr_rewrite_const_initializer(concat_expr, type, rhs_init);
@@ -663,7 +665,7 @@ bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *
 					FOREACH(ConstInitializer *, element, rhs_init->init_array.elements)
 					{
 						ASSERT_SPAN(right, element->kind == CONST_INIT_ARRAY_VALUE);
-						element->init_array_value.index += len_lhs;
+						element->init_array_value.index += (ArrayIndex)len_lhs;
 						vec_add(inits, element);
 					}
 					expr_rewrite_const_initializer(concat_expr, type, const_init_new_array(type, inits));
