@@ -113,7 +113,7 @@ Expr *sema_resolve_string_ident(SemaContext *context, Expr *inner, bool report_m
 bool sema_analyse_asm(SemaContext *context, AsmInlineBlock *block, Ast *asm_stmt);
 bool sema_expr_analyse_sprintf(SemaContext *context, Expr *expr, Expr *format_string, Expr **args, unsigned num_args);
 
-bool sema_bit_assignment_check(SemaContext *context, Expr *right, Decl *member);
+bool sema_bit_assignment_check(SemaContext *context, Expr *right, Decl *member, bool *failed_ref);
 CondResult sema_check_comp_time_bool(SemaContext *context, Expr *expr);
 
 bool sema_expr_check_assign(SemaContext *context, Expr *expr, bool *failed_ref);
@@ -130,7 +130,7 @@ Decl *sema_analyse_parameterized_identifier(SemaContext *c, Path *decl_path, con
                                             Expr **params, bool *was_recursive_ref, SourceSpan invocation_span);
 bool sema_parameterized_type_is_found(SemaContext *context, Path *decl_path, const char *name, SourceSpan span);
 Type *sema_resolve_type_get_func(Signature *signature, CallABI abi);
-INLINE bool sema_set_abi_alignment(SemaContext *context, Type *type, AlignSize *result);
+INLINE bool sema_set_abi_alignment(SemaContext *context, Type *type, AlignSize *result, bool as_member);
 INLINE bool sema_set_alloca_alignment(SemaContext *context, Type *type, AlignSize *result);
 INLINE void sema_display_deprecated_warning_on_use(SemaContext *context, Decl *decl, SourceSpan span);
 bool sema_expr_analyse_ct_concat(SemaContext *context, Expr *concat_expr, Expr *left, Expr *right, bool *failed_ref);
@@ -170,14 +170,28 @@ INLINE bool sema_check_left_right_const(SemaContext *context, Expr *left, Expr *
 	return true;
 }
 
-INLINE bool sema_set_abi_alignment(SemaContext *context, Type *type, AlignSize *result)
+INLINE bool sema_set_abi_alignment(SemaContext *context, Type *type, AlignSize *result, bool as_member)
 {
+	type = type->canonical;
 	if (type_is_func_ptr(type))
 	{
 		*result = type_abi_alignment(type_voidptr);
 		return true;
 	}
 	if (!sema_resolve_type_decl(context, type)) return false;
+	if (as_member)
+	{
+		while (type->type_kind == TYPE_TYPEDEF)
+		{
+			if (type_is_simd(type)) goto DONE;
+			type = type->decl->distinct->type->canonical;
+		}
+		if (type_kind_is_any_vector(type->type_kind))
+		{
+			type = type->array.base;
+		}
+	}
+DONE:;
 	*result = type_abi_alignment(type);
 	return true;
 }
@@ -201,7 +215,7 @@ INLINE Attr* attr_find_kind(Attr **attrs, AttributeType attr_type)
 INLINE void sema_display_deprecated_warning_on_use(SemaContext *context, Decl *decl, SourceSpan span)
 {
 	ASSERT(decl->resolve_status == RESOLVE_DONE);
-	if (!decl->resolved_attributes || !decl->attrs_resolved || !decl->attrs_resolved->deprecated) return;
+	if (!decl_is_deprecated(decl)) return;
 	if (context->call_env.ignore_deprecation) return;
 	const char *msg = decl->attrs_resolved->deprecated;
 
