@@ -35,7 +35,6 @@ void context_change_scope_with_flags(SemaContext *context, ScopeFlags flags)
 
 	unsigned label_start = new_label_scope ? last_local : context->active_scope.label_start;
 	context->active_scope = (DynamicScope) {
-			.scope_id = ++context->scope_id,
 			.allow_dead_code = false,
 			.is_dead = scope_is_dead,
 			.is_poisoned = scope_is_poisoned,
@@ -47,10 +46,6 @@ void context_change_scope_with_flags(SemaContext *context, ScopeFlags flags)
 			.defer_start = parent_defer,
 			.flags = flags,
 	};
-	if (context->scope_id == 0)
-	{
-		FATAL_ERROR("Too many scopes.");
-	}
 }
 
 const char *context_filename(SemaContext *context)
@@ -584,11 +579,13 @@ void sema_print_inline(SemaContext *context, SourceSpan original)
 {
 	if (!context) return;
 	InliningSpan *inlined_at = context->inlined_at;
+	SourceSpan last_span = INVALID_SPAN;
 	while (inlined_at)
 	{
-		if (inlined_at->span.a != original.a)
+		if (inlined_at->span.a != original.a && inlined_at->span.a != last_span.a)
 		{
 			sema_note_prev_at(inlined_at->span, "Inlined from here.");
+			last_span = inlined_at->span;
 		}
 		inlined_at = inlined_at->prev;
 	}
