@@ -23,6 +23,8 @@
 #define NO_ARENA 0
 #define MAX_VECTOR_WIDTH 65536
 #define MAX_STACK_OBJECT_SIZE (256 * 1024)
+#define MAX_MACRO_ITERATIONS 0xFFFFFF
+#define DEFAULT_MAX_MACRO_ITERATIONS 0xFFFFF
 #define DEFAULT_VECTOR_WIDTH 4096
 #define DEFAULT_STACK_OBJECT_SIZE 64
 #define MAX_ARRAY_SIZE INT64_MAX
@@ -79,18 +81,35 @@
 #define UNUSED __attribute__((unused))
 #define NORETURN __attribute__((noreturn))
 #define INLINE __attribute__((always_inline)) static inline
+#define FORMAT_STR
+#define FORMAT(__X__, __Y__) __attribute__((format (printf, __X__, __Y__)))
 #elif defined(_MSC_VER)
 #define FALLTHROUGH ((void)0)
 #define INLINE static __forceinline
 #define NORETURN __declspec(noreturn)
 #define UNUSED
 #define PACK( __Declaration__ ) __pragma( pack(push, 1) ) __Declaration__ __pragma( pack(pop))
+// On msvc when using /analyze flag it is possible to have printf-style format strings compiler warnings
+// by using the SAL annotations, but __attribute__((format (printf, x, y))) syntax is not supported.
+#define FORMAT(__X__, __Y__)
+#if _MSC_VER >= 1400
+	#include <sal.h>
+	#if _MSC_VER > 1400
+		#define FORMAT_STR _Printf_format_string_
+	#else
+		#define FORMAT_STR __format_string
+	#endif
+#else
+	#define FORMAT_STR
+#endif
 #else
 #define PACK(__Declaration__) __Declaration__
 #define INLINE static inline
 #define FALLTHROUGH ((void)0)
 #define UNUSED
 #define NORETURN
+#define FORMAT_STR
+#define FORMAT(__X__, __Y__)
 #endif
 
 #define INFO_LOG(_string, ...) \
@@ -110,11 +129,11 @@
   } while (0)
 #endif
 
-#define FATAL_ERROR(_string, ...) do { error_exit("\xe2\x9a\xa0\xef\xb8\x8f The compiler encountered an unexpected error: \"%s\".\n\n" \
+#define FATAL_ERROR(_format, ...) do { error_exit("\xe2\x9a\xa0\xef\xb8\x8f The compiler encountered an unexpected error: \"" _format "\".\n\n" \
  "- Function: %s(...)\n" \
  "- Source file: %s:%d\n\n" \
- "\xf0\x9f\x99\x8f Please consider taking the time to file an issue on GitHub, so that we can get it fixed:\n\n" \
- "https://github.com/c3lang/c3c/issues/new so that we can get it fixed.", _string, __func__, __FILE__, __LINE__, ##__VA_ARGS__); } while(0)
+ "\xf0\x9f\x99\x8f Please consider taking the time to file an issue on GitHub (https://github.com/c3lang/c3c/issues/new), so that we can get it fixed.\n", \
+  ##__VA_ARGS__, __func__, __FILE__, __LINE__); } while(0)
 
 
 #define ASSERT(_condition) do { if (!(_condition)) { FATAL_ERROR("Violated assert: " #_condition); } } while (0)
@@ -126,7 +145,7 @@
 #define UNSUPPORTED do { error_exit("Unsupported functionality"); } while (0)
 
 #define TEST_ASSERT(condition_, string_) while (!(condition_)) { FATAL_ERROR(string_); }
-#define TEST_ASSERTF(condition_, string_, ...) while (!(condition_)) { char* str_ = str_printf(string_, __VA_ARGS__); FATAL_ERROR(str_); }
+#define TEST_ASSERTF(condition_, format_, ...) while (!(condition_)) { FATAL_ERROR(format_, __VA_ARGS__); }
 
 #define EXPECT(_string, _value, _expected) \
  do { long long __tempval1 = _value; long long __tempval2 = _expected; \
@@ -134,4 +153,5 @@
 
 void evprintf(const char *format, va_list list);
 void eprintf(const char *format, ...);
-NORETURN void error_exit(const char *format, ...);
+NORETURN FORMAT(1, 2) void error_exit(FORMAT_STR const char *format, ...);
+
